##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Hookable object support
"""

class _py_hookable(object):
    __slots__ = ('_original', '_implementation')
    
    def __init__(self, *args, **kw):
        if len(args) == 0 and 'implementation' in kw:
            args = (kw.pop('implementation'),)
        if kw:
            raise TypeError('Unknown keyword arguments')
        if len(args) != 1:
            raise TypeError('Exactly one argument required')
        self._original = self._implementation = args[0]

    original = property(lambda self: self._original)
    implementation = property(lambda self: self._implementation)

    def sethook(self, new_callable):
        old, self._implementation = self._implementation, new_callable
        return old

    def reset(self):
        self._implementation = self._original

    def __call__(self, *args, **kw):
        return self._implementation(*args, **kw)

hookable = _py_hookable

try:
    from ._zope_hookable import hookable
except ImportError: #pragma NO COVER
    pass



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.hookable package
"""
import os
import platform

from setuptools import setup, find_packages, Extension, Feature

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

Cwrapper = Feature(
    "C wrapper",
    standard = True,
    ext_modules=[Extension("zope.hookable._zope_hookable",
                            [os.path.join('src', 'zope', 'hookable',
                                        "_zope_hookable.c")
                            ],
                            extra_compile_args=['-g']),
                ],
)
py_impl = getattr(platform, 'python_implementation', lambda: None)
is_pypy = py_impl() == 'PyPy'

# Jython cannot build the C optimizations, while on PyPy they are
# anti-optimizations (the C extension compatibility layer is known-slow,
# and defeats JIT opportunities).
if is_pypy:
    features = {}
else:
    features = {'Cwrapper': Cwrapper}

setup(name='zope.hookable',
      version = '4.0.4',
      url='http://svn.zope.org/zope.hookable',
      license='ZPL 2.1',
      description='Zope hookable',
      author='Zope Foundation and Contributors',
      author_email='zope-dev@zope.org',
      long_description=(read('README.rst') + '\n\n' +
                        read('CHANGES.rst')),
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Zope Public License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Framework :: Zope3",
        "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      features=features,
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['zope',],
      install_requires=['setuptools'],
      include_package_data=True,
      zip_safe=False,
      test_suite='zope.hookable.tests.test_hookable.test_suite',
      extras_require = {
        'docs': ['Sphinx'],
        'testing': ['nose', 'coverage'],
        'test': ['zope.testing'],
      },
)



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test the hookable support Extension
"""
import unittest

class PyHookableTests(unittest.TestCase):

    def _callFUT(self, *args, **kw):
        from zope.hookable import _py_hookable
        return _py_hookable(*args, **kw)

    def test_before_hook(self):
        def _foo():
            return 'FOO'
        hooked = self._callFUT(_foo)
        self.assertTrue(hooked.original is _foo)
        self.assertTrue(hooked.implementation is _foo)
        self.assertEqual(hooked(), 'FOO')

    def test_after_hook(self):
        def _foo():
            return 'FOO'
        def _bar():
            return 'BAR'
        hooked = self._callFUT(_foo)
        old = hooked.sethook(_bar)
        self.assertTrue(old is _foo)
        self.assertTrue(hooked.original is _foo)
        self.assertTrue(hooked.implementation is _bar)
        self.assertEqual(hooked(), 'BAR')

    def test_after_hook_and_reset(self):
        def _foo():
            return 'FOO'
        def _bar():
            return 'BAR'
        hooked = self._callFUT(_foo)
        old = hooked.sethook(_bar)
        hooked.reset()
        self.assertTrue(old is _foo)
        self.assertTrue(hooked.original is _foo)
        self.assertTrue(hooked.implementation is _foo)
        self.assertEqual(hooked(), 'FOO')

    def test_original_cannot_be_deleted(self):
        def _foo():
            return 'FOO'
        hooked = self._callFUT(_foo)
        def _try():
            del hooked.original
        self.assertRaises((TypeError, AttributeError), _try)

    def test_implementation_cannot_be_deleted(self):
        def _foo():
            return 'FOO'
        hooked = self._callFUT(_foo)
        def _try():
            del hooked.implementation
        self.assertRaises((TypeError, AttributeError), _try)

    def test_no_args(self):
        self.assertRaises(TypeError, self._callFUT)

    def test_too_many_args(self):
        def _foo():
            return 'FOO'
        self.assertRaises(TypeError, self._callFUT, _foo, _foo)

    def test_w_implementation_kwarg(self):
        def _foo():
            return 'FOO'
        hooked = self._callFUT(implementation=_foo)
        self.assertTrue(hooked.original is _foo)
        self.assertTrue(hooked.implementation is _foo)
        self.assertEqual(hooked(), 'FOO')

    def test_w_unknown_kwarg(self):
        def _foo():
            return 'FOO'
        self.assertRaises(TypeError, self._callFUT, nonesuch=_foo)


class HookableTests(PyHookableTests):

    def _callFUT(self, *args, **kw):
        from zope.hookable import hookable
        return hookable(*args, **kw)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(PyHookableTests),
        unittest.makeSuite(HookableTests),
    ))
